@@ -380,6 +380,9 @@ NvCfgBool nvCfgIsPrimaryDevice(NvCfgDeviceHandle handle,
  * responsibility to free the returned array of strings. On success, 
  * NVCFG_TRUE will be returned and serials will be assigned.  On failure,
  * NVCFG_FALSE will be returned.
+ *
+ * Note that this function is deprecated and will always return an empty array
+ * on recent drivers, since QuadroPlex devices are no longer supported.
  */
 
 NvCfgBool nvCfgGetTeslaSerialNumbers(char ***serials);
@@ -450,4 +453,22 @@ NvCfgBool nvCfgFlashGSyncDevice(NvCfgGSyncHandle handle, int format,
  */
 NvCfgBool nvCfgDumpDisplayPortAuxLog(NvCfgDeviceHandle handle);
 
+/*
+ * nvCfgEnableUvmPersistence() - Enable UVM Persistence mode.
+ * On success, zero will be returned. On failure, non zero
+ * value will be returned.
+ * This API is for internal use only and will be removed
+ * in a future release.
+ */
+unsigned int nvCfgEnableUVMPersistence(NvCfgDeviceHandle handle);
+
+/*
+ * nvCfgDisableUVMPersistence() - Disable UVM Persistence mode.
+ * On success, zero will be returned. On failure, non-zero
+ * value will be returned.
+ * This API is for internal use only and will be removed
+ * in a future release.
+ */
+unsigned int nvCfgDisableUVMPersistence(NvCfgDeviceHandle handle);
+
 #endif /* __NVIDIA_CFG__ */
